@@ -2,7 +2,7 @@
 
 set -e
 
-git clone https://gitlab.freedesktop.org/pwithnall/malcontent.git
+git clone --depth 1 --no-tags https://gitlab.freedesktop.org/pwithnall/malcontent.git
 meson subprojects download --sourcedir malcontent
 rm malcontent/subprojects/*.wrap
 mv malcontent/subprojects/ .
