@@ -18,6 +18,7 @@
  *
  * Authors:
  *  - Felipe Borges <felipeborges@gnome.org>
+ *  - Georges Basile Stavracas Neto <georges@endlessos.org>
  *  - Philip Withnall <withnall@endlessm.com>
  */
 
@@ -29,13 +30,16 @@
 
 #define ARROW_SIZE 20
 
+#define MCT_TYPE_CAROUSEL_LAYOUT (mct_carousel_layout_get_type ())
+G_DECLARE_FINAL_TYPE (MctCarouselLayout, mct_carousel_layout, MCT, CAROUSEL_LAYOUT, GtkLayoutManager)
+
 struct _MctCarouselItem {
-  GtkRadioButton parent;
+  GtkButton parent;
 
   gint page;
 };
 
-G_DEFINE_TYPE (MctCarouselItem, mct_carousel_item, GTK_TYPE_RADIO_BUTTON)
+G_DEFINE_TYPE (MctCarouselItem, mct_carousel_item, GTK_TYPE_BUTTON)
 
 GtkWidget *
 mct_carousel_item_new (void)
@@ -43,28 +47,36 @@ mct_carousel_item_new (void)
   return g_object_new (MCT_TYPE_CAROUSEL_ITEM, NULL);
 }
 
+void
+mct_carousel_item_set_child (MctCarouselItem *self,
+                             GtkWidget       *child)
+{
+  g_return_if_fail (MCT_IS_CAROUSEL_ITEM (self));
+
+  gtk_button_set_child (GTK_BUTTON (self), child);
+}
+
 static void
 mct_carousel_item_class_init (MctCarouselItemClass *klass)
 {
+  gtk_widget_class_set_css_name (GTK_WIDGET_CLASS (klass), "carousel-item");
 }
 
 static void
 mct_carousel_item_init (MctCarouselItem *self)
 {
-  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (self), FALSE);
-  gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (self)),
-             "carousel-item");
 }
 
 struct _MctCarousel {
-  GtkRevealer parent;
+  AdwBin parent;
+
+  GtkRevealer *revealer;
 
   GList *children;
   gint visible_page;
   MctCarouselItem *selected_item;
   GtkWidget *last_box;
   GtkWidget *arrow;
-  gint arrow_start_x;
 
   /* Widgets */
   GtkStack *stack;
@@ -74,7 +86,7 @@ struct _MctCarousel {
   GtkStyleProvider *provider;
 };
 
-G_DEFINE_TYPE (MctCarousel, mct_carousel, GTK_TYPE_REVEALER)
+G_DEFINE_TYPE (MctCarousel, mct_carousel, ADW_TYPE_BIN)
 
 enum {
   ITEM_ACTIVATED,
@@ -91,70 +103,46 @@ mct_carousel_item_get_x (MctCarouselItem *item,
 {
   GtkWidget *widget, *parent;
   gint width;
-  gint dest_x;
+  gdouble dest_x;
+  graphene_point_t p;
 
-  parent = GTK_WIDGET (carousel->stack);
+  parent = GTK_WIDGET (carousel->revealer);
   widget = GTK_WIDGET (item);
 
-  width = gtk_widget_get_allocated_width (widget);
-  if (!gtk_widget_translate_coordinates (widget,
-                                         parent,
-                                         width / 2,
-                                         0,
-                                         &dest_x,
-                                         NULL))
+  width = gtk_widget_get_width (widget);
+  if (!gtk_widget_compute_point (widget, parent,
+                                 &GRAPHENE_POINT_INIT (width / 2, 0),
+                                 &p))
     return 0;
 
+  dest_x = p.x;
+
   return CLAMP (dest_x - ARROW_SIZE,
                 0,
-                gtk_widget_get_allocated_width (parent));
+                gtk_widget_get_width (parent));
 }
 
 static void
 mct_carousel_move_arrow (MctCarousel *self)
 {
-  GtkStyleContext *context;
   gchar *css;
   gint end_x;
-  GtkSettings *settings;
-  gboolean animations;
 
   if (!self->selected_item)
     return;
 
   end_x = mct_carousel_item_get_x (self->selected_item, self);
 
-  context = gtk_widget_get_style_context (self->arrow);
   if (self->provider)
-    gtk_style_context_remove_provider (context, self->provider);
+    gtk_style_context_remove_provider_for_display (gtk_widget_get_display (self->arrow), self->provider);
   g_clear_object (&self->provider);
 
-  settings = gtk_widget_get_settings (GTK_WIDGET (self));
-  g_object_get (settings, "gtk-enable-animations", &animations, NULL);
-
-  /* Animate the arrow movement if animations are enabled. Otherwise,
-   * jump the arrow to the right location instantly. */
-  if (animations)
-    {
-      css = g_strdup_printf ("@keyframes arrow_keyframes-%d-%d {\n"
-                             "  from { margin-left: %dpx; }\n"
-                             "  to { margin-left: %dpx; }\n"
-                             "}\n"
-                             "* {\n"
-                             "  animation-name: arrow_keyframes-%d-%d;\n"
-                             "}\n",
-                             self->arrow_start_x, end_x,
-                             self->arrow_start_x, end_x,
-                             self->arrow_start_x, end_x);
-    }
-  else
-    {
-      css = g_strdup_printf ("* { margin-left: %dpx }", end_x);
-    }
-
+  css = g_strdup_printf (".carousel-arrow { margin-left: %dpx; }", end_x);
   self->provider = GTK_STYLE_PROVIDER (gtk_css_provider_new ());
-  gtk_css_provider_load_from_data (GTK_CSS_PROVIDER (self->provider), css, -1, NULL);
-  gtk_style_context_add_provider (context, self->provider, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+  gtk_css_provider_load_from_string (GTK_CSS_PROVIDER (self->provider), css);
+  gtk_style_context_add_provider_for_display (gtk_widget_get_display (self->arrow),
+                                              self->provider,
+                                              GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 
   g_free (css);
 }
@@ -234,10 +222,7 @@ mct_carousel_select_item (MctCarousel     *self,
     }
 
   if (self->selected_item != NULL)
-    {
-      page_changed = (self->selected_item->page != item->page);
-      self->arrow_start_x = mct_carousel_item_get_x (self->selected_item, self);
-    }
+    page_changed = (self->selected_item->page != item->page);
 
   self->selected_item = item;
   self->visible_page = item->page;
@@ -300,59 +285,44 @@ mct_carousel_goto_next_page (GtkWidget *button,
   mct_carousel_select_item_at_index (self, self->visible_page * ITEMS_PER_PAGE);
 }
 
-static void
-mct_carousel_add (GtkContainer *container,
-                  GtkWidget    *widget)
+void
+mct_carousel_add (MctCarousel     *self,
+                  MctCarouselItem *item)
 {
-  MctCarousel *self = MCT_CAROUSEL (container);
   gboolean last_box_is_full;
 
-  if (!MCT_IS_CAROUSEL_ITEM (widget))
-    {
-      GTK_CONTAINER_CLASS (mct_carousel_parent_class)->add (container, widget);
-      return;
-    }
+  g_return_if_fail (MCT_IS_CAROUSEL (self));
+  g_return_if_fail (MCT_IS_CAROUSEL_ITEM (item));
 
-  gtk_style_context_add_class (gtk_widget_get_style_context (widget), "menu");
-  gtk_button_set_relief (GTK_BUTTON (widget), GTK_RELIEF_NONE);
-
-  self->children = g_list_append (self->children, widget);
-  MCT_CAROUSEL_ITEM (widget)->page = get_last_page_number (self);
-  if (self->selected_item != NULL)
-    gtk_radio_button_join_group (GTK_RADIO_BUTTON (widget), GTK_RADIO_BUTTON (self->selected_item));
-  g_signal_connect (widget, "button-press-event", G_CALLBACK (on_item_toggled), self);
+  self->children = g_list_append (self->children, item);
+  item->page = get_last_page_number (self);
+  g_signal_connect (item, "clicked", G_CALLBACK (on_item_toggled), self);
 
   last_box_is_full = ((g_list_length (self->children) - 1) % ITEMS_PER_PAGE == 0);
   if (last_box_is_full)
     {
       g_autofree gchar *page = NULL;
 
-      page = g_strdup_printf ("%d", MCT_CAROUSEL_ITEM (widget)->page);
-      self->last_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-      gtk_widget_show (self->last_box);
+      page = g_strdup_printf ("%d", item->page);
+      self->last_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 18);
+      gtk_widget_set_hexpand (self->last_box, TRUE);
       gtk_widget_set_valign (self->last_box, GTK_ALIGN_CENTER);
+      gtk_box_set_homogeneous (GTK_BOX (self->last_box), TRUE);
       gtk_stack_add_named (self->stack, self->last_box, page);
     }
 
-  gtk_widget_show_all (widget);
-  gtk_box_pack_start (GTK_BOX (self->last_box), widget, TRUE, FALSE, 10);
+  gtk_box_append (GTK_BOX (self->last_box), GTK_WIDGET (item));
 
   update_buttons_visibility (self);
 }
 
-static void
-destroy_widget_cb (GtkWidget *widget,
-                   gpointer   user_data)
-{
-  gtk_widget_destroy (widget);
-}
-
 void
 mct_carousel_purge_items (MctCarousel *self)
 {
-  gtk_container_forall (GTK_CONTAINER (self->stack),
-                        destroy_widget_cb,
-                        NULL);
+  GtkWidget *child;
+
+  while ((child = gtk_widget_get_first_child (GTK_WIDGET (self->stack))) != NULL)
+    gtk_stack_remove (self->stack, child);
 
   g_list_free (self->children);
   self->children = NULL;
@@ -386,7 +356,6 @@ mct_carousel_class_init (MctCarouselClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *wclass = GTK_WIDGET_CLASS (klass);
-  GtkContainerClass *container_class = GTK_CONTAINER_CLASS (klass);
 
   gtk_widget_class_set_template_from_resource (wclass,
                                                "/org/freedesktop/MalcontentControl/ui/carousel.ui");
@@ -395,13 +364,14 @@ mct_carousel_class_init (MctCarouselClass *klass)
   gtk_widget_class_bind_template_child (wclass, MctCarousel, go_back_button);
   gtk_widget_class_bind_template_child (wclass, MctCarousel, go_next_button);
   gtk_widget_class_bind_template_child (wclass, MctCarousel, arrow);
+  gtk_widget_class_bind_template_child (wclass, MctCarousel, revealer);
 
   gtk_widget_class_bind_template_callback (wclass, mct_carousel_goto_previous_page);
   gtk_widget_class_bind_template_callback (wclass, mct_carousel_goto_next_page);
 
-  object_class->dispose = mct_carousel_dispose;
+  gtk_widget_class_set_layout_manager_type (wclass, MCT_TYPE_CAROUSEL_LAYOUT);
 
-  container_class->add = mct_carousel_add;
+  object_class->dispose = mct_carousel_dispose;
 
   signals[ITEM_ACTIVATED] =
       g_signal_new ("item-activated",
@@ -414,19 +384,6 @@ mct_carousel_class_init (MctCarouselClass *klass)
                     MCT_TYPE_CAROUSEL_ITEM);
 }
 
-static void
-on_size_allocate (MctCarousel *self)
-{
-  if (self->selected_item == NULL)
-    return;
-
-  if (gtk_stack_get_transition_running (self->stack))
-    return;
-
-  self->arrow_start_x = mct_carousel_item_get_x (self->selected_item, self);
-  mct_carousel_move_arrow (self);
-}
-
 static void
 on_transition_running (MctCarousel *self)
 {
@@ -445,13 +402,12 @@ mct_carousel_init (MctCarousel *self)
   gtk_css_provider_load_from_resource (GTK_CSS_PROVIDER (provider),
                                        "/org/freedesktop/MalcontentControl/ui/carousel.css");
 
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                             provider,
-                                             GTK_STYLE_PROVIDER_PRIORITY_APPLICATION - 1);
+  gtk_style_context_add_provider_for_display (gdk_display_get_default (),
+                                              provider,
+                                              GTK_STYLE_PROVIDER_PRIORITY_APPLICATION - 1);
 
   g_object_unref (provider);
 
-  g_signal_connect_swapped (self->stack, "size-allocate", G_CALLBACK (on_size_allocate), self);
   g_signal_connect_swapped (self->stack, "notify::transition-running", G_CALLBACK (on_transition_running), self);
 }
 
@@ -460,3 +416,77 @@ mct_carousel_get_item_count (MctCarousel *self)
 {
   return g_list_length (self->children);
 }
+
+void
+mct_carousel_set_revealed (MctCarousel *self,
+                           gboolean     revealed)
+{
+  g_return_if_fail (MCT_IS_CAROUSEL (self));
+
+  gtk_revealer_set_reveal_child (self->revealer, revealed);
+}
+
+struct _MctCarouselLayout {
+  GtkLayoutManager parent;
+};
+
+G_DEFINE_FINAL_TYPE (MctCarouselLayout, mct_carousel_layout, GTK_TYPE_LAYOUT_MANAGER)
+
+static void
+mct_carousel_layout_measure (GtkLayoutManager *layout_manager,
+                             GtkWidget        *widget,
+                             GtkOrientation    orientation,
+                             int               for_size,
+                             int              *minimum,
+                             int              *natural,
+                             int              *minimum_baseline,
+                             int              *natural_baseline)
+{
+  MctCarousel *carousel;
+
+  g_assert (MCT_IS_CAROUSEL (widget));
+
+  carousel = MCT_CAROUSEL (widget);
+
+  gtk_widget_measure (GTK_WIDGET (carousel->revealer),
+                      orientation, for_size,
+                      minimum, natural,
+                      minimum_baseline, natural_baseline);
+}
+
+static void
+mct_carousel_layout_allocate (GtkLayoutManager *layout_manager,
+                              GtkWidget        *widget,
+                              int               width,
+                              int               height,
+                              int               baseline)
+{
+  MctCarousel *carousel;
+
+  g_assert (MCT_IS_CAROUSEL (widget));
+
+  carousel = MCT_CAROUSEL (widget);
+  gtk_widget_allocate (GTK_WIDGET (carousel->revealer), width, height, baseline, NULL);
+
+  if (carousel->selected_item == NULL)
+    return;
+
+  if (gtk_stack_get_transition_running (carousel->stack))
+    return;
+
+  mct_carousel_move_arrow (carousel);
+}
+
+static void
+mct_carousel_layout_class_init (MctCarouselLayoutClass *klass)
+{
+  GtkLayoutManagerClass *layout_manager_class = GTK_LAYOUT_MANAGER_CLASS (klass);
+
+  layout_manager_class->measure = mct_carousel_layout_measure;
+  layout_manager_class->allocate = mct_carousel_layout_allocate;
+}
+
+static void
+mct_carousel_layout_init (MctCarouselLayout *self)
+{
+}
